@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: getcolor.c,v 1.32 2011/10/08 00:07:41 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: getcolor.c,v 1.39 2016/09/17 04:52:42 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - getcolor.c */
@@ -90,7 +90,7 @@ palettes_differ(t_sm_palette *p1, t_sm_palette *p2)
 
 	if (p1->gradient_num != p2->gradient_num)
 	    return 1;
-	for(i=0; i<p1->gradient_num; ++i) {
+	for (i=0; i<p1->gradient_num; ++i) {
 	    if (p1->gradient[i].pos != p2->gradient[i].pos)
 		return 1;
 	    if (p1->gradient[i].col.r != p2->gradient[i].col.r)
@@ -236,6 +236,7 @@ color_components_from_gray(double gray, rgb_color *color)
 	gray = 1.0;
 
     switch(sm_palette.colorMode) {
+    default:	/* Can't happen */
     case SMPAL_COLOR_MODE_GRAY:
 	color->r = color->g = color->b = pow(gray, 1.0/sm_palette.gamma);
 	return;  /* all done, no color space transformation needed  */
@@ -267,9 +268,6 @@ color_components_from_gray(double gray, rgb_color *color)
 	if (color->b > 1.0) color->b = 1.0; if (color->b < 0.0) color->b = 0.0;
 	}	
 	break;
-    default:
-	fprintf(stderr, "%s:%d ooops: Unknown colorMode '%c'.\n",
-		__FILE__, __LINE__, (char)(sm_palette.colorMode));
     }
 }
 
@@ -291,6 +289,7 @@ rgb1_from_gray(double gray, rgb_color *color)
 
     /* transform to RGB if necessary */
     switch(sm_palette.cmodel) {
+    default:
     case C_MODEL_RGB:
 	break;
     case C_MODEL_HSV:
@@ -305,9 +304,6 @@ rgb1_from_gray(double gray, rgb_color *color)
     case C_MODEL_XYZ:
 	CIEXYZ_2_RGB(color);
 	break;
-    default:
-	fprintf(stderr, "%s:%d ooops: Unknown color model '%c'\n",
-		__FILE__, __LINE__, (char)(sm_palette.cmodel));
     }
 }
 
@@ -346,27 +342,44 @@ rgb1maxcolors_from_gray(double gray, rgb_color *color)
 double 
 quantize_gray( double gray )
 {
-    double degray = floor(gray * sm_palette.use_maxcolors)
-			/ (sm_palette.use_maxcolors-1);
+    double qgray = floor(gray * sm_palette.use_maxcolors)
+		 / (sm_palette.use_maxcolors-1);
 
     if (sm_palette.colorMode == SMPAL_COLOR_MODE_GRADIENT) {
 	int j;
-	if ((sm_palette.gradient_num <= 2) && (degray == 0))
-	    ; /* Backward compatibility with common case of 1 segment */
+	gradient_struct *g = sm_palette.gradient;
+	double small_interval = 1. / sm_palette.use_maxcolors;
+
+	/* Backward compatibility with common case of 1 segment */
+	if ((sm_palette.gradient_num <= 2) && (qgray == 0))
+	    ;
+
+	/* All palette segments are large compared to the sampling interval.
+	 * Simple truncation of gray is good enough.
+	 */
+	else if (sm_palette.smallest_gradient_interval > small_interval)
+	    ;
+
+	/* There is at least one palette segment that is smaller than the sampling
+	 * interval. Earlier versions of quantize_gray() handled this case poorly.
+	 * This version isn't perfect either.  In particular it fails to handle the
+	 * converse problematic case where qgray is inside some small interval but
+	 * the true gray value is not.  This causes a color from the narrow segment 
+	 *  to be applied incorrectly to neighoring segments as well.
+	 */
 	else for (j=0; j<sm_palette.gradient_num; j++) {
-	    if ((gray >= sm_palette.gradient[j].pos)
-	    &&  (gray <  sm_palette.gradient[j+1].pos)) {
-		if ((degray < sm_palette.gradient[j].pos)
-		||  (degray > sm_palette.gradient[j+1].pos))
-		    degray = (sm_palette.gradient[j].pos
-			    + sm_palette.gradient[j+1].pos) / 2.;
-	    }
-	    if (gray < sm_palette.gradient[j+1].pos)
+	    /* Does the true gray value lie in this interval? */
+	    if ((gray >= g[j].pos) &&  (gray <  g[j+1].pos)) {
+		/* See if it is so small that truncation missed it */
+		if ((g[j+1].pos - g[j].pos) < small_interval)
+		    qgray = (g[j].pos + g[j+1].pos) / 2.;
 		break;
+	    }
+
 	}
     }
 
-    return degray;	
+    return qgray;	
 }
 
 
@@ -397,7 +410,7 @@ get_max_dev(rgb_color *colors, int j, double limit)
     double sg = (colors[j].g - g) / j;
     double sb = (colors[j].b - b) / j;
 
-    for(i=1; i<j; ++i) {
+    for (i=1; i<j; ++i) {
 	double dx = i;
 
 	rdev = fabs(sr*dx + r - colors[i].r);
@@ -446,8 +459,7 @@ is_extremum(rgb_color left, rgb_color mid, rgb_color right)
 #define GROW_GRADIENT(n) do {						\
     if(cnt == gradient_size) {						\
 	gradient_size += (n);						\
-	gradient = (gradient_struct*)					\
-	    realloc(gradient, gradient_size * sizeof(gradient_struct));	\
+	gradient = realloc(gradient, gradient_size * sizeof(*gradient));\
     }									\
 } while(0)
 
@@ -500,13 +512,12 @@ approximate_palette(t_sm_palette *palette, int samples,
     ++cnt;
     color_components_from_gray(1.0 / samples, colors + 1);
 
-    for(i = 0; i < samples; ++i) {
-	for(j = 2; i + j <= samples; ++j) {
+    for (i = 0; i < samples; ++i) {
+	for (j = 2; i + j <= samples; ++j) {
 	    gray = ((double) (i + j)) / samples;
 	    if (j == colors_size) {
 	        colors_size += 50;
-		colors = (rgb_color *)
-		    realloc(colors, colors_size*sizeof(gradient_struct));
+		colors = realloc(colors, colors_size*sizeof(*colors));
 	    }
 	    color_components_from_gray(gray, colors + j);
 
@@ -697,8 +708,9 @@ GetColorValueFromFormula(int formula, double x)
 	   sm_palette, first item---search for "t_sm_palette sm_palette = "
 	 */
     default:
-	fprintf(stderr, "Fatal: undefined color formula (can be 0--%i)\n", sm_palette.colorFormulae - 1);
-	exit(1);
+	/* Cannot happen! */
+	FPRINTF((stderr, "gnuplot:  invalid palette rgbformula"));
+	x = 0;
     }
     if (x <= 0)
 	return 0;
@@ -858,5 +870,17 @@ HSV_2_RGB(rgb_color *col)
 }
 #undef CONSTRAIN
 
+/*
+ * Support for user-callable rgb = hsv2rgb(h,s,v)
+ */
+unsigned int 
+hsv2rgb ( rgb_color *color )
+{
+    HSV_2_RGB(color);
+
+    return    ((unsigned int)(255.*color->r) << 16) 
+	    + ((unsigned int)(255.*color->g) << 8)
+	    + ((unsigned int)(255.*color->b));
+}
 
 /* eof getcolor.c */
