@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.99.2.9 2017/04/13 03:13:11 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.118 2017/03/05 05:55:25 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - hidden3d.c */
@@ -134,7 +134,7 @@ static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.99.2.9 2017/04/13 03:13
 
 /* The actual configuration is stored in these variables, modifiable
  * at runtime through 'set hidden3d' options */
-static int hiddenBacksideLinetypeOffset = BACKSIDE_LINETYPE_OFFSET;
+       int hiddenBacksideLinetypeOffset = BACKSIDE_LINETYPE_OFFSET;
 static long hiddenTriangleLinesdrawnPattern = TRIANGLE_LINESDRAWN_PATTERN;
 static int hiddenHandleUndefinedPoints = HANDLE_UNDEFINED_POINTS;
 static int hiddenShowAlternativeDiagonal = SHOW_ALTERNATIVE_DIAGONAL;
@@ -303,8 +303,8 @@ static void color_edges __PROTO((long int new_edge, long int old_edge,
 				 int style_above, int style_below));
 static void build_networks __PROTO((struct surface_points * plots,
 				    int pcount));
-int compare_edges_by_zmin __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
-int compare_polys_by_zmax __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
+static int compare_edges_by_zmin __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
+static int compare_polys_by_zmax __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
 static void sort_edges_by_z __PROTO((void));
 static void sort_polys_by_z __PROTO((void));
 static TBOOLEAN get_plane __PROTO((p_polygon p, t_plane plane));
@@ -528,7 +528,7 @@ store_vertex (
 
 #ifdef HIDDEN3D_VAR_PTSIZE
     /* Store pointer back to original point */
-    /* Needed to support variable pointsize */
+    /* Needed to support variable pointsize or pointtype */
     thisvert->original = point;
 #endif
 	
@@ -1068,14 +1068,14 @@ build_networks(struct surface_points *plots, int pcount)
 
 	/* count 'curves' (i.e. isolines) and vertices in this plot */
 	nverts = 0;
-	if(this_plot->plot_type == FUNC3D) {
+	if (this_plot->plot_type == FUNC3D) {
 	    ncrvs = 0;
-	    for(icrvs = this_plot->iso_crvs;
+	    for (icrvs = this_plot->iso_crvs;
 		icrvs; icrvs = icrvs->next) {
 		ncrvs++;
 	    }
 	    nverts += ncrvs * crvlen;
-	} else if(this_plot->plot_type == DATA3D) {
+	} else if (this_plot->plot_type == DATA3D) {
 	    ncrvs = this_plot->num_iso_read;
 	    if (this_plot->has_grid_topology)
 		nverts += ncrvs * crvlen;
@@ -1087,7 +1087,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    nverts += icrvs->p_count;
 	    }
 	} else {
-	    graph_error("Plot type is neither function nor data");
+	    /* Cannot happen */
 	    return;
 	}
 
@@ -1237,8 +1237,7 @@ build_networks(struct surface_points *plots, int pcount)
 			else
 			    labelpoint.CRD_COLOR = label->textcolor.lt;
 			
-			thisvertex = store_vertex(&labelpoint, 
-				&(this_plot->lp_properties), color_from_column);
+			thisvertex = store_vertex(&labelpoint, lp, color_from_column);
 			if (thisvertex < 0)
 			    continue;
 			(vlist+thisvertex)->label = label;
@@ -1534,8 +1533,7 @@ build_networks(struct surface_points *plots, int pcount)
 /* Sort the elist in order of growing zmax. Uses qsort on an array of
  * plist indices, and then fills in the 'next' fields in struct
  * polygon to store the resulting order inside the plist */
-/* HBB 20010720: removed 'static' to avoid HP-sUX gcc bug */
-int
+static int
 compare_edges_by_zmin(SORTFUNC_ARGS p1, SORTFUNC_ARGS p2)
 {
     return SIGN(vlist[elist[*(const long *) p1].v2].z
@@ -1573,8 +1571,7 @@ sort_edges_by_z()
     free(sortarray);
 }
 
-/* HBB 20010720: removed 'static' to avoid HP-sUX gcc bug */
-int
+static int
 compare_polys_by_zmax(SORTFUNC_ARGS p1, SORTFUNC_ARGS p2)
 {
     return (SIGN(plist[*(const long *) p1].zmax
@@ -1667,7 +1664,7 @@ draw_vertex(p_vertex v)
     p_type = v->lp_style->p_type;
 
     TERMCOORD(v, x, y);
-    if ((p_type >= -1 || p_type == PT_CHARACTER) && !clip_point(x,y)) {
+    if ((p_type >= -1 || p_type == PT_CHARACTER || p_type == PT_VARIABLE) && !clip_point(x,y)) {
 	struct t_colorspec *tc = &(v->lp_style->pm3d_color);
 
 	if (v->label)  {
@@ -1680,7 +1677,7 @@ draw_vertex(p_vertex v)
 	    struct lp_style_type style = *(v->lp_style);
 	    load_linetype(&style, (int)v->real_z);
 	    tc = &style.pm3d_color;
-	    apply_pm3dcolor(tc, term);
+	    apply_pm3dcolor(tc);
 	}
 	else if (tc->type == TC_RGB && tc->lt == LT_COLORFROMCOLUMN)
 	    set_rgbcolor_var((unsigned int)v->real_z);
@@ -1697,9 +1694,13 @@ draw_vertex(p_vertex v)
 #endif
 
 	if (p_type == PT_CHARACTER)
-	    (term->put_text)(x, y, (char *)(&(v->lp_style->p_char)));
+	    (term->put_text)(x, y, v->lp_style->p_char);
+#ifdef HIDDEN3D_VAR_PTSIZE
+	else if (p_type == PT_VARIABLE)
+	    (term->point)(x, y, (int)(v->original->CRD_PTTYPE) - 1);
+#endif
 	else
-	    (term->point)(x,y, p_type);
+	    (term->point)(x, y, p_type);
 
 	/* vertex has been drawn --> flag it as done */
 	v->lp_style = NULL;
@@ -1790,7 +1791,7 @@ draw_edge(p_edge e, p_vertex v1, p_vertex v2)
 	    /* a pointer to the immediately following field e->arrow_properties.   */
 	    lp_style_type *lp = e->lp;
 	    arrow_style_type *as = (arrow_style_type *)(&lp[1]);
-	    apply_3dhead_properties(as);
+	    apply_head_properties(as);
 	}
     }
 
@@ -1968,8 +1969,8 @@ in_front(
     grid_y_low = coord_to_treecell(ymin);
     grid_y_high = coord_to_treecell(ymax);
 
-    for (grid_x = grid_x_low; grid_x <= grid_x_high; grid_x ++)
-	for (grid_y = grid_y_low; grid_y <= grid_y_high; grid_y ++)
+    for (grid_x = grid_x_low; grid_x <= grid_x_high; grid_x++)
+	for (grid_y = grid_y_low; grid_y <= grid_y_high; grid_y++)
 	    for (listhead = quadtree[grid_x][grid_y];
 		 listhead >= 0;
 		 listhead = qlist[listhead].next)
@@ -2295,7 +2296,7 @@ plot3d_hidden(struct surface_points *plots, int pcount)
     if (! edges.end) {
 	/* No drawable edges found. Free all storage and bail out. */
 	term_hidden_line_removal();
-	graph_error("*All* edges undefined or out of range, thus no plot.");
+	int_error(NO_CARET, "*All* edges undefined or out of range, thus no plot.");
     }
 
     if (! polygons.end) {
