@@ -1,5 +1,5 @@
 /*
- * $Id: internal.h,v 1.19.2.2 2010/09/19 02:31:57 sfeam Exp $
+ * $Id: internal.h,v 1.27 2016/09/19 04:40:30 sfeam Exp $
  */
 
 /* GNUPLOT - internal.h */
@@ -55,6 +55,7 @@ void f_pushd __PROTO((union argument *x));
 void f_pop __PROTO((union argument *x));
 void f_call __PROTO((union argument *x));
 void f_calln __PROTO((union argument *x));
+void f_sum __PROTO((union argument *x));
 void f_lnot __PROTO((union argument *x));
 void f_bnot __PROTO((union argument *x));
 void f_lor __PROTO((union argument *x));
@@ -69,6 +70,8 @@ void f_gt __PROTO((union argument *x));
 void f_lt __PROTO((union argument *x));
 void f_ge __PROTO((union argument *x));
 void f_le __PROTO((union argument *x));
+void f_leftshift __PROTO((union argument *x));
+void f_rightshift __PROTO((union argument *x));
 void f_plus __PROTO((union argument *x));
 void f_minus __PROTO((union argument *x));
 void f_mult __PROTO((union argument *x));
@@ -82,13 +85,17 @@ void f_eqs __PROTO((union argument *x));
 void f_nes __PROTO((union argument *x));
 void f_gprintf __PROTO((union argument *x));
 void f_range __PROTO((union argument *x));
+void f_index __PROTO((union argument *x));
+void f_cardinality __PROTO((union argument *x));
 void f_sprintf __PROTO((union argument *x));
 void f_strlen __PROTO((union argument *x));
 void f_strstrt __PROTO((union argument *x));
 void f_system __PROTO((union argument *x));
+void f_word __PROTO((union argument *x));
 void f_words __PROTO((union argument *x));
 void f_strftime __PROTO((union argument *x));
 void f_strptime __PROTO((union argument *x));
+void f_time __PROTO((union argument *x));
 void f_assign __PROTO((union argument *x));
 void f_value __PROTO((union argument *x));
 
