@@ -1,5 +1,5 @@
 /*
- * $Id: mousecmn.h,v 1.17.2.2 2017/02/25 16:13:31 markisch Exp $
+ * $Id: mousecmn.h,v 1.19.2.1 2017/09/06 18:43:26 sfeam Exp $
  */
 
 /* GNUPLOT - mousecnm.h */
@@ -78,8 +78,11 @@ enum {
 
 
 /* the status of the shift, ctrl and alt keys
-*/
-enum { Mod_Shift = (1), Mod_Ctrl = (1 << 1), Mod_Alt = (1 << 2) };
+ * Mod_Opt is used by the "bind" mechanism to indicate that the
+ * Ctrl or Alt key is allowed but not required
+ */
+enum { Mod_Shift = (1), Mod_Ctrl = (1 << 1), Mod_Alt = (1 << 2),
+       Mod_Opt = (1 << 3) };
 
 /* the below depends on the ascii character set lying in the
  * range from 0 to 255 (below 1000) */
