@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.66 2011/09/29 18:39:01 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.112.2.5 2017/09/20 01:40:35 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -38,17 +38,22 @@ static char *RCSid() { return RCSid("$Id: parse.c,v 1.66 2011/09/29 18:39:01 sfe
 
 #include "alloc.h"
 #include "command.h"
+#include "datablock.h"
 #include "eval.h"
 #include "help.h"
 #include "util.h"
 
-/* protection mechanism for parsing string followed by + or - sign */
+/* Protection mechanism for trying to parse a string followed by a + or - sign.
+ * Also suppresses an undefined variable message if an unrecognized token
+ * is encountered during try_to_get_string().
+ */
+TBOOLEAN string_result_only = FALSE;
 static int parse_recursion_level;
-static TBOOLEAN string_result_only = FALSE;
 
 /* Exported globals: the current 'dummy' variable names */
 char c_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1];
 char set_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1] = { "x", "y" };
+int  fit_dummy_var[MAX_NUM_VAR];
 TBOOLEAN scanning_range_in_progress = FALSE;
 
 /* This is used by plot_option_using() */
@@ -57,8 +62,10 @@ int at_highest_column_used = -1;
 /* This is checked by df_readascii() */
 TBOOLEAN parse_1st_row_as_headers = FALSE;
 
+/* This is used by df_open() and df_readascii() */
+udvt_entry *df_array = NULL;
+
 /* Iteration structures used for bookkeeping */
-/* Iteration can be nested so long as different iterators are used */
 t_iterator * plot_iterator = NULL;
 t_iterator * set_iterator = NULL;
 
@@ -75,6 +82,7 @@ static void accept_exclusive_OR_expression __PROTO((void));
 static void accept_AND_expression __PROTO((void));
 static void accept_equality_expression __PROTO((void));
 static void accept_relational_expression __PROTO((void));
+static void accept_bitshift_expression __PROTO((void));
 static void accept_additive_expression __PROTO((void));
 static void accept_multiplicative_expression __PROTO((void));
 static void parse_primary_expression __PROTO((void));
@@ -86,13 +94,21 @@ static void parse_exclusive_OR_expression __PROTO((void));
 static void parse_AND_expression __PROTO((void));
 static void parse_equality_expression __PROTO((void));
 static void parse_relational_expression __PROTO((void));
+static void parse_bitshift_expression __PROTO((void));
 static void parse_additive_expression __PROTO((void));
 static void parse_multiplicative_expression __PROTO((void));
 static void parse_unary_expression __PROTO((void));
 static void parse_sum_expression __PROTO((void));
 static int  parse_assignment_expression __PROTO((void));
+static int  parse_array_assignment_expression __PROTO((void));
 static int is_builtin_function __PROTO((int t_num));
 
+static void set_up_columnheader_parsing __PROTO((struct at_entry *previous ));
+
+static TBOOLEAN no_iteration __PROTO((t_iterator *));
+static void reevaluate_iteration_limits __PROTO((t_iterator *iter));
+static void reset_iteration __PROTO((t_iterator *iter));
+
 /* Internal variables: */
 
 static struct at_type *at = NULL;
@@ -122,6 +138,13 @@ real_expression()
 }
 
 
+void
+parse_reset_after_error()
+{
+    string_result_only = FALSE;
+    parse_recursion_level = 0;
+}
+
 /* JW 20051126:
  * Wrapper around const_express() called by try_to_get_string().
  * Disallows top level + and - operators.
@@ -152,12 +175,19 @@ const_express(struct value *valptr)
     if (undefined) {
 	int_error(tkn, "undefined value");
     }
+
+    if (valptr->type == ARRAY) {
+	/* Make sure no one tries to free it later */
+	valptr->type = NOTDEFINED;
+	int_error(NO_CARET, "const_express: unsupported array operation");
+    }
+
     return (valptr);
 }
 
-/* Used by plot2d/plot3d/fit:
- * Parse an expression that may return a string or may return a constant or may
- * be a dummy function using dummy variables x, y, ...
+/* Used by plot2d/plot3d/stats/fit:
+ * Parse an expression that may return a filename string, a datablock name,
+ * a constant, or a dummy function using dummy variables x, y, ...
  * If any dummy variables are present, set (*atptr) to point to an action table
  * corresponding to the parsed expression, and return NULL.
  * Otherwise evaluate the expression and return a string if there is one.
@@ -170,18 +200,31 @@ string_or_express(struct at_type **atptr)
     int i;
     TBOOLEAN has_dummies;
 
+    static char *array_placeholder = "@@";
     static char* str = NULL;
     free(str);
     str = NULL;
+    df_array = NULL;
+
+    if (atptr)
+	*atptr = NULL;
 
     if (END_OF_COMMAND)
 	int_error(c_token, "expression expected");
 
-    if (isstring(c_token)) {
-	if (atptr)
-	    *atptr = NULL;
-	str = try_to_get_string();
+    /* parsing for datablocks */
+    if (equals(c_token,"$"))
+	return parse_datablock_name();
+
+    if (isstring(c_token) && (str = try_to_get_string()))
 	return str;
+
+    /* If this is a bare array name for an existing array, store a pointer */
+    /* for df_open() to use.  "@@" is a magic pseudo-filename passed to    */
+    /* df_open() that tells it to use the stored pointer.                  */
+    if (type_udv(c_token) == ARRAY && !equals(c_token+1, "[")) {
+	df_array = add_udv(c_token++);
+	return array_placeholder;
     }
 
     /* parse expression */
@@ -203,8 +246,15 @@ string_or_express(struct at_type **atptr)
 	struct value val;
 
 	evaluate_at(at, &val);
-	if (!undefined && val.type == STRING)
-	    str = val.v.string_val;
+	if (!undefined && val.type == STRING) {
+	    /* prevent empty string variable from treated as special file '' or "" */
+	    if (*val.v.string_val == '\0') {
+		free(val.v.string_val);
+		str = strdup(" ");
+	    } else {
+		str = val.v.string_val;
+	    }
+	}
     }
 
     /* prepare return */
@@ -245,8 +295,8 @@ perm_at()
     size_t len;
 
     (void) temp_at();
-    len = sizeof(struct at_type) +
-     (at->a_count - MAX_AT_LEN) * sizeof(struct at_entry);
+    len = sizeof(struct at_type)
+	+ (at->a_count - MAX_AT_LEN) * sizeof(struct at_entry);
     at_ptr = (struct at_type *) gp_realloc(at, len, "perm_at");
     at = NULL;			/* invalidate at pointer */
     return (at_ptr);
@@ -360,11 +410,19 @@ accept_equality_expression()
 static void
 accept_relational_expression()
 {
-    accept_additive_expression();
+    accept_bitshift_expression();
     parse_relational_expression();
 }
 
 
+static void
+accept_bitshift_expression()
+{
+    accept_additive_expression();
+    parse_bitshift_expression();
+}
+
+
 static void
 accept_additive_expression()
 {
@@ -383,29 +441,103 @@ accept_multiplicative_expression()
 static int
 parse_assignment_expression()
 {
-    /* Check for assignment operator */
+    /* Check for assignment operator Var = <expr> */
     if (isletter(c_token) && equals(c_token + 1, "=")) {
+
 	/* push the variable name */
 	union argument *foo = add_action(PUSHC);
 	char *varname = NULL;
 	m_capture(&varname,c_token,c_token);
 	foo->v_arg.type = STRING;
 	foo->v_arg.v.string_val = varname;
+
+	/* push a dummy variable that would be the index if this were an array */
+	/* FIXME: It would be nice to hide this from "show at" */
+	foo = add_action(PUSHC);
+	foo->v_arg.type = NOTDEFINED;
+
+	/* push the expression whose value it will get */
 	c_token += 2;
-	/* and the expression whose value it will get */
 	parse_expression();
-	/* and the actual assignment operation */
+
+	/* push the actual assignment operation */
 	(void) add_action(ASSIGN);
 	return 1;
     }
+
+    /* Check for assignment to an array element Array[<expr>] = <expr> */
+    if (parse_array_assignment_expression())
+	return 1;
+
     return 0;
 }
 
+/*
+ * If an array assignment is the first thing on a command line it is handled by
+ * the separate routine array_assignment().
+ * Here we catch assignments that are embedded in an expression.
+ * Examples:
+ *	print A[2] = foo
+ *	A[1] = A[2] = A[3] = 0
+ */
+static int
+parse_array_assignment_expression()
+{
+    /* Check for assignment to an array element Array[<expr>] = <expr> */
+    if (isletter(c_token) && equals(c_token+1, "[")) {
+	char *varname = NULL;
+	union argument *foo;
+	int save_action, save_token;
+
+	/* Quick check for the most common false positives */
+	/* i.e. other constructs that begin with "name["   */
+	if (equals(c_token,"sum") && equals(c_token+3, "="))
+	    return 0;
+	if (equals(c_token+3, ":"))
+	    return 0;
+	if (equals(c_token+3, "]") && !equals(c_token+4, "="))
+	    return 0;
+
+	/* Save state of the action table and the command line */
+	save_action = at->a_count;
+	save_token = c_token;
+
+	/* push the array name */
+	m_capture(&varname,c_token,c_token);
+	foo = add_action(PUSHC);
+	foo->v_arg.type = STRING;
+	foo->v_arg.v.string_val = varname;
+
+	/* push the index */
+	c_token += 2;
+	parse_expression();
+
+	/* If this wasn't really an array element assignment, back out. */
+	/* NB: Depending on what we just parsed, this may leak memory.  */
+	if (!equals(c_token, "]") || !equals(c_token+1, "=")) {
+	    c_token = save_token;
+	    at->a_count = save_action;
+	    free(varname);
+	    return 0;
+	}
+
+	/* Now we evaluate the expression whose value it will get */
+	c_token += 2;
+	parse_expression();
+
+	/* push the actual assignment operation */
+	(void) add_action(ASSIGN);
+	return 1;
+    }
+
+    return 0;
+}
 
 /* add action table entries for primary expressions, i.e. either a
- * parenthesized expression, a variable names, a numeric constant, a
+ * parenthesized expression, a variable name, a numeric constant, a
  * function evaluation, a power operator or postfix '!' (factorial)
- * expression */
+ * expression.
+ * Sep 2016 cardinality expression |Array| */
 static void
 parse_primary_expression()
 {
@@ -426,19 +558,50 @@ parse_primary_expression()
     } else if (equals(c_token, "$")) {
 	struct value a;
 
-	if (!isanumber(++c_token))
-	    int_error(c_token, "Column number expected");
-	convert(&a, c_token++);
-	if (a.type != INTGR || a.v.int_val < 0)
-	    int_error(c_token, "Positive integer expected");
-	if (at_highest_column_used < a.v.int_val)
-	    at_highest_column_used = a.v.int_val;
-	add_action(DOLLARS)->v_arg = a;
+	c_token++;
+	if (!isanumber(c_token)) {
+	    if (equals(c_token+1, "[")) {
+		struct udvt_entry *datablock_udv;
+		c_token--;
+		datablock_udv = get_udv_by_name(parse_datablock_name());
+		if (!datablock_udv)
+		    int_error(c_token-2,"No such datablock");
+		add_action(PUSH)->udv_arg = datablock_udv;
+	    } else
+		int_error(c_token, "Column number or datablock line expected");
+	} else if (equals(c_token,"N")) {
+	    /* $N == pseudocolumn -3 means "last column" */
+	    c_token++;
+	    Ginteger(&a, -3);
+	    at_highest_column_used = -3;
+	    add_action(DOLLARS)->v_arg = a;
+	} else {
+	    convert(&a, c_token++);
+	    if (a.type != INTGR || a.v.int_val < 0)
+		int_error(c_token, "Positive integer expected");
+	    if (at_highest_column_used < a.v.int_val)
+		at_highest_column_used = a.v.int_val;
+	    add_action(DOLLARS)->v_arg = a;
+	}
+    } else if (equals(c_token, "|")) {
+	struct udvt_entry *udv;
+	c_token++;
+	if (equals(c_token,"$")) {
+	    udv = get_udv_by_name(parse_datablock_name());
+	    if (!udv)
+		int_error(c_token-1, "no such datablock");
+	} else {
+	    udv = add_udv(c_token++);
+	    if (udv->udv_value.type != ARRAY)
+		int_error(c_token-1, "not an array");
+	}
+	add_action(PUSH)->udv_arg = udv;
+	if (!equals(c_token, "|"))
+	    int_error(c_token, "'|' expected");
+	c_token++;
+	add_action(CARDINALITY);
     } else if (isanumber(c_token)) {
-	/* work around HP 9000S/300 HP-UX 9.10 cc limitation ... */
-	/* HBB 20010724: use this code for all platforms, then */
 	union argument *foo = add_action(PUSHC);
-
 	convert(&(foo->v_arg), c_token);
 	c_token++;
     } else if (isletter(c_token)) {
@@ -450,19 +613,23 @@ parse_primary_expression()
 	    struct value num_params;
 	    num_params.type = INTGR;
 
-	    if (whichfunc) {
-#ifdef BACKWARDS_COMPATIBLE
+#if (1)	    /* DEPRECATED */
+	    if (whichfunc && (strcmp(ft[whichfunc].f_name,"defined")==0)) {
 		/* Deprecated syntax:   if (defined(foo)) ...  */
 		/* New syntax:          if (exists("foo")) ... */
-		if (strcmp(ft[whichfunc].f_name,"defined")==0) {
-		    struct udvt_entry *udv = add_udv(c_token+2);
-		    union argument *foo = add_action(PUSHC);
-		    foo->v_arg.type = INTGR;
-		    foo->v_arg.v.int_val = udv->udv_undef ? 0 : 1;
-		    c_token += 4;  /* skip past "defined ( <foo> ) " */
-		    return;
-		}
+		struct udvt_entry *udv = add_udv(c_token+2);
+		union argument *foo = add_action(PUSHC);
+		foo->v_arg.type = INTGR;
+		if (udv->udv_value.type == NOTDEFINED)
+		    foo->v_arg.v.int_val = 0;
+		else
+		    foo->v_arg.v.int_val = 1;
+		c_token += 4;  /* skip past "defined ( <foo> ) " */
+		return;
+	    }
 #endif
+
+	    if (whichfunc) {
 		c_token += 2;	/* skip fnc name and '(' */
 		parse_expression(); /* parse fnc argument */
 		num_params.v.int_val = 1;
@@ -481,27 +648,13 @@ parse_primary_expression()
 		if (!strcmp(ft[whichfunc].f_name,"sprintf"))
 		    add_action(PUSHC)->v_arg = num_params;
 
-		/* "words(s)" is implemented as "word(s,-1)" */
-		if (!strcmp(ft[whichfunc].f_name,"words")) {
-		    num_params.v.int_val = -1;
+		/* v4 timecolumn only had 1 param; v5 has 2. Accept either */
+		if (!strcmp(ft[whichfunc].f_name,"timecolumn"))
 		    add_action(PUSHC)->v_arg = num_params;
-		}
 
-		/* column("string") means we expect the first row of */
-		/* a data file to contain headers rather than data.  */
+		/* The column() function has side effects requiring special handling */
 		if (!strcmp(ft[whichfunc].f_name,"column")) {
-		    struct at_entry *previous = &(at->actions[at->a_count-1]);
-		    if (previous->index == PUSHC
-		    &&  previous->arg.v_arg.type == STRING)
-			parse_1st_row_as_headers = TRUE;
-		}
-		/* This allows plot ... using (column(N)) title columnhead */
-		if (!strcmp(ft[whichfunc].f_name,"column")) {
-		    struct at_entry *previous = &(at->actions[at->a_count-1]);
-		    if (previous->index == PUSHC
-		    &&  previous->arg.v_arg.type == INTGR)
-			if (at_highest_column_used < previous->arg.v_arg.v.int_val)
-			    at_highest_column_used = previous->arg.v_arg.v.int_val;
+		    set_up_columnheader_parsing( &(at->actions[at->a_count-1]) );
 		}
 
 		(void) add_action(whichfunc);
@@ -535,9 +688,11 @@ parse_primary_expression()
 	    if (equals(c_token, c_dummy_var[0])) {
 		c_token++;
 		add_action(PUSHD1)->udf_arg = dummy_func;
+		fit_dummy_var[0]++;
 	    } else if (equals(c_token, c_dummy_var[1])) {
 		c_token++;
 		add_action(PUSHD2)->udf_arg = dummy_func;
+		fit_dummy_var[1]++;
 	    } else {
 		int i, param = 0;
 
@@ -550,6 +705,7 @@ parse_primary_expression()
 			c_token++;
 			add_action(PUSHC)->v_arg = num_params;
 			add_action(PUSHD)->udf_arg = dummy_func;
+			fit_dummy_var[i]++;
 			break;
 		    }
 		}
@@ -574,50 +730,71 @@ parse_primary_expression()
 	/* this dynamically allocated string will be freed by free_at() */
 	m_quote_capture(&(foo->v_arg.v.string_val), c_token, c_token);
 	c_token++;
-    } else
+    } else {
 	int_error(c_token, "invalid expression ");
-
-    /* add action code for ! (factorial) operator */
-    while (equals(c_token, "!")) {
-	c_token++;
-	(void) add_action(FACTORIAL);
-    }
-    /* add action code for ** operator */
-    if (equals(c_token, "**")) {
-	c_token++;
-	parse_unary_expression();
-	(void) add_action(POWER);
     }
 
-    /* Parse and add actions for range specifier applying to previous entity.
-     * Currently only used to generate substrings, but could also be used to
-     * extract vector slices.
-     */
-    if (equals(c_token, "[")) {
-	/* handle '*' or empty start of range */
-	if (equals(++c_token,"*") || equals(c_token,":")) {
-	    union argument *empty = add_action(PUSHC);
-	    empty->v_arg.type = INTGR;
-	    empty->v_arg.v.int_val = 1;
-	    if (equals(c_token,"*"))
+    /* The remaining operators are postfixes and can be stacked, e.g. */
+    /* Array[i]**2, so we may have to loop to catch all of them.      */
+    while (TRUE) {
+
+	/* add action code for ! (factorial) operator */
+	if (equals(c_token, "!")) {
+	    c_token++;
+	    (void) add_action(FACTORIAL);
+	}
+
+	/* add action code for ** operator */
+	else if (equals(c_token, "**")) {
+	    c_token++;
+	    parse_unary_expression();
+	    (void) add_action(POWER);
+	}
+
+	/* Parse and add actions for range specifier applying to previous entity.
+	 * Currently the [beg:end] form is used to generate substrings, but could
+	 * also be used to extract vector slices.  The [i] form is used to index
+	 * arrays, but could also be a shorthand for extracting a single-character
+	 * substring.
+	 */
+	else if (equals(c_token, "[") && !isanumber(c_token-1)) {
+	    /* handle '*' or empty start of range */
+	    if (equals(++c_token,"*") || equals(c_token,":")) {
+		union argument *empty = add_action(PUSHC);
+		empty->v_arg.type = INTGR;
+		empty->v_arg.v.int_val = 1;
+		if (equals(c_token,"*"))
+		    c_token++;
+	    } else
+		parse_expression();
+
+	    /* handle array indexing (single value in square brackets) */
+	    if (equals(c_token, "]")) {
 		c_token++;
-	} else
-	    parse_expression();
-	if (!equals(c_token, ":"))
-	    int_error(c_token, "':' expected");
-	/* handle '*' or empty end of range */
-	if (equals(++c_token,"*") || equals(c_token,"]")) {
-	    union argument *empty = add_action(PUSHC);
-	    empty->v_arg.type = INTGR;
-	    empty->v_arg.v.int_val = 65535; /* should be MAXINT */
-	    if (equals(c_token,"*"))
-		c_token++;
-	} else
-	    parse_expression();
-	if (!equals(c_token, "]"))
-	    int_error(c_token, "']' expected");
-	c_token++;
-	(void) add_action(RANGE);
+		(void) add_action(INDEX);
+		continue;
+	    }
+
+	    if (!equals(c_token, ":"))
+		int_error(c_token, "':' expected");
+	    /* handle '*' or empty end of range */
+	    if (equals(++c_token,"*") || equals(c_token,"]")) {
+		union argument *empty = add_action(PUSHC);
+		empty->v_arg.type = INTGR;
+		empty->v_arg.v.int_val = 65535; /* should be INT_MAX */
+		if (equals(c_token,"*"))
+		    c_token++;
+	    } else
+		parse_expression();
+	    if (!equals(c_token, "]"))
+		int_error(c_token, "']' expected");
+	    c_token++;
+	    (void) add_action(RANGE);
+
+	/* Whatever this is, it isn't another postfix operator */
+	} else {
+	    break;
+	}
     }
 }
 
@@ -771,10 +948,9 @@ parse_relational_expression()
      * operators */
 
     while (TRUE) {
-	/* I hate "else if" statements */
 	if (equals(c_token, ">")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(GT);
 	} else if (equals(c_token, "<")) {
 	    /*  Workaround for * in syntax of range constraints  */
@@ -782,15 +958,15 @@ parse_relational_expression()
 		break;
 	    }
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(LT);
 	} else if (equals(c_token, ">=")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(GE);
 	} else if (equals(c_token, "<=")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(LE);
 	} else
 	    break;
@@ -800,6 +976,26 @@ parse_relational_expression()
 
 
 
+static void
+parse_bitshift_expression()
+{
+    /* create action codes for << and >> operators */
+    while (TRUE) {
+	if (equals(c_token, "<<")) {
+	    c_token++;
+	    accept_additive_expression();
+	    (void) add_action(LEFTSHIFT);
+	} else if (equals(c_token, ">>")) {
+	    c_token++;
+	    accept_additive_expression();
+	    (void) add_action(RIGHTSHIFT);
+	} else
+	    break;
+    }
+}
+
+
+
 static void
 parse_additive_expression()
 {
@@ -811,9 +1007,9 @@ parse_additive_expression()
 	    (void) add_action(CONCATENATE);
 	/* If only string results are wanted
 	 * do not accept '-' or '+' at the top level. */
-	} else if (string_result_only && parse_recursion_level == 1)
+	} else if (string_result_only && parse_recursion_level == 1) {
 	    break;
-	else if (equals(c_token, "+")) {
+	} else if (equals(c_token, "+")) {
 	    c_token++;
 	    accept_multiplicative_expression();
 	    (void) add_action(PLUS);
@@ -865,9 +1061,20 @@ parse_unary_expression()
 	parse_unary_expression();
 	(void) add_action(BNOT);
     } else if (equals(c_token, "-")) {
+	struct at_entry *previous;
 	c_token++;
 	parse_unary_expression();
-	(void) add_action(UMINUS);
+	/* Collapse two operations PUSHC <pos-const> + UMINUS
+	 * into a single operation PUSHC <neg-const>
+	 */
+	previous = &(at->actions[at->a_count-1]);
+	if (previous->index == PUSHC &&  previous->arg.v_arg.type == INTGR) {
+	    previous->arg.v_arg.v.int_val = -previous->arg.v_arg.v.int_val;
+	} else if (previous->index == PUSHC &&  previous->arg.v_arg.type == CMPLX) {
+	    previous->arg.v_arg.v.cmplx_val.real = -previous->arg.v_arg.v.cmplx_val.real;
+	    previous->arg.v_arg.v.cmplx_val.imag = -previous->arg.v_arg.v.cmplx_val.imag;
+	} else
+	    (void) add_action(UMINUS);
     } else if (equals(c_token, "+")) {	/* unary + is no-op */
 	c_token++;
 	parse_unary_expression();
@@ -876,6 +1083,34 @@ parse_unary_expression()
 }
 
 
+/*
+ * Syntax: set link {x2|y2} {via <expression1> inverse <expression2>}
+ * Create action code tables for the functions linking primary and secondary axes.
+ * expression1 maps primary coordinates into the secondary coordinate space.
+ * expression2 maps secondary coordinates into the primary coordinate space.
+ */
+void
+parse_link_via( struct udft_entry *udf )
+{
+    int start_token;
+    
+    /* Caller left us pointing at "via" or "inverse" */
+    c_token++;
+    start_token = c_token;
+    if (END_OF_COMMAND)
+	int_error(c_token,"Missing expression");
+
+    /* Save action table for the linkage mapping */
+    dummy_func = udf;
+    free_at(udf->at);
+    udf->at = perm_at();
+    dummy_func = NULL;
+
+    /* Save the mapping expression itself */
+    m_capture(&(udf->definition), start_token, c_token - 1);
+}
+
+
 /* create action code for 'sum' expressions */
 static void
 parse_sum_expression()
@@ -964,6 +1199,8 @@ add_udv(int t_num)
 {
     char varname[MAX_ID_LEN+1];
     copy_str(varname, t_num, MAX_ID_LEN);
+    if (token[t_num].length > MAX_ID_LEN-1)
+	int_warn(t_num, "truncating variable name that is too long");
     return add_udv_by_name(varname);
 }
 
@@ -1015,194 +1252,316 @@ is_builtin_function(int t_num)
     return (0);
 }
 
+/*
+ * Test for the existence of a function without triggering errors
+ * Return values:
+ *   0  no such function is defined
+ *  -1  built-in function
+ *   1  user-defined function
+ */
+int
+is_function(int t_num)
+{
+    struct udft_entry **udf_ptr = &first_udf;
+
+    if (is_builtin_function(t_num))
+	return -1;
+
+    while (*udf_ptr) {
+	if (equals(t_num, (*udf_ptr)->udf_name))
+	    return 1;
+	udf_ptr = &((*udf_ptr)->next_udf);
+    }
+
+    return 0;
+}
+
 /* Look for iterate-over-plot constructs, of the form
  *    for [<var> = <start> : <end> { : <increment>}] ...
  * If one (or more) is found, an iterator structure is allocated and filled
  * and a pointer to that structure is returned.
  * The pointer is NULL if no "for" statements are found.
+ * If the iteration limits are constants, store them as is.
+ * If they are given as expressions, store an action table for the expression.
  */
 t_iterator *
 check_for_iteration()
 {
-    char *errormsg = "Expecting iterator \tfor [<var> = <start> : <end>]\n\t\t\tor\tfor [<var> in \"string of words\"]";
+    char *errormsg = "Expecting iterator \tfor [<var> = <start> : <end> {: <incr>}]\n\t\t\tor\tfor [<var> in \"string of words\"]";
     int nesting_depth = 0;
     t_iterator *iter = NULL;
+    t_iterator *prev = NULL;
     t_iterator *this_iter = NULL;
+    TBOOLEAN no_parent = FALSE;
 
     /* Now checking for iteration parameters */
     /* Nested "for" statements are supported, each one corresponds to a node of the linked list */
     while (equals(c_token, "for")) {
 	struct udvt_entry *iteration_udv = NULL;
+	t_value original_udv_value;
 	char *iteration_string = NULL;
 	int iteration_start;
 	int iteration_end;
 	int iteration_increment = 1;
 	int iteration_current;
 	int iteration = 0;
-	TBOOLEAN empty_iteration;
+	struct at_type *iteration_start_at = NULL;
+	struct at_type *iteration_end_at = NULL;
 
 	c_token++;
 	if (!equals(c_token++, "[") || !isletter(c_token))
 	    int_error(c_token-1, errormsg);
 	iteration_udv = add_udv(c_token++);
+	original_udv_value = iteration_udv->udv_value;
+	iteration_udv->udv_value.type = NOTDEFINED;
 
 	if (equals(c_token, "=")) {
 	    c_token++;
-	    iteration_start = int_expression();
+	    if (isanumber(c_token) && equals(c_token+1,":")) {
+		/* Save the constant value only */
+		iteration_start = int_expression();
+	    } else {
+		/* Save the expression as well as the value */
+		struct value v;
+		iteration_start_at = perm_at();
+		if (no_parent) {
+		    iteration_start = 0;
+		} else {
+		    evaluate_at(iteration_start_at, &v);
+		    iteration_start = real(&v);
+		}
+	    }
 	    if (!equals(c_token++, ":"))
 	    	int_error(c_token-1, errormsg);
-	    iteration_end = int_expression();
+	    if (equals(c_token,"*")) {
+		iteration_end = INT_MAX;
+		c_token++;
+	    } else if (isanumber(c_token) && (equals(c_token+1,":") || equals(c_token+1,"]"))) {
+		/* Save the constant value only */
+		iteration_end = int_expression();
+	    } else {
+		/* Save the expression as well as the value */
+		struct value v;
+		iteration_end_at = perm_at();
+		if (no_parent) {
+		    iteration_end = 0;
+		} else {
+		    evaluate_at(iteration_end_at, &v);
+		    iteration_end = real(&v);
+		}
+	    }
 	    if (equals(c_token,":")) {
 	    	c_token++;
 	    	iteration_increment = int_expression();
+		if (iteration_increment == 0)
+		    int_error(c_token-1, errormsg);
 	    }
 	    if (!equals(c_token++, "]"))
 	    	int_error(c_token-1, errormsg);
-	    if (iteration_udv->udv_undef == FALSE)
-		gpfree_string(&(iteration_udv->udv_value));
+	    gpfree_array(&(iteration_udv->udv_value));
+	    gpfree_string(&(iteration_udv->udv_value));
 	    Ginteger(&(iteration_udv->udv_value), iteration_start);
-	    iteration_udv->udv_undef = FALSE;
 	}
 	else if (equals(c_token++, "in")) {
-	    iteration_string = try_to_get_string();
-	    if (!iteration_string)
+	    /* Assume this is a string-valued expression. */
+	    /* It might be worth treating a string constant as a special case */
+	    struct value v;
+	    iteration_start_at = perm_at();
+	    evaluate_at(iteration_start_at, &v);
+	    if (v.type != STRING)
 	    	int_error(c_token-1, errormsg);
 	    if (!equals(c_token++, "]"))
 	    	int_error(c_token-1, errormsg);
+	    iteration_string = v.v.string_val;
 	    iteration_start = 1;
 	    iteration_end = gp_words(iteration_string);
-	    if (iteration_udv->udv_undef == FALSE)
-	    	gpfree_string(&(iteration_udv->udv_value));
+	    gpfree_array(&(iteration_udv->udv_value));
+	    gpfree_string(&(iteration_udv->udv_value));
 	    Gstring(&(iteration_udv->udv_value), gp_word(iteration_string, 1));
-	    iteration_udv->udv_undef = FALSE;
 	}
 	else /* Neither [i=B:E] or [s in "foo"] */
 	    int_error(c_token-1, errormsg);
 
 	iteration_current = iteration_start;
-	
-	empty_iteration = iteration_udv 
-	    && ((iteration_end - iteration_start) * iteration_increment < 0);
-        
-	/* allocating a node of the linked list and initializing its fields */
-	/* iterating just once is the same as not iterating at all, 
-	 * so we skip building the node in that case */
-	if (iteration_increment 
-	&& (iteration_start != iteration_end)
-	&& (abs(iteration_end - iteration_start) >= abs(iteration_increment))) {
-	    this_iter = gp_alloc(sizeof(t_iterator), "iteration linked list");
-	    this_iter->iteration_udv = iteration_udv; 
-	    this_iter->iteration_string = iteration_string;
-	    this_iter->iteration_start = iteration_start;
-	    this_iter->iteration_end = iteration_end;
-	    this_iter->iteration_increment = iteration_increment;
-	    this_iter->iteration_current = iteration_current;
-	    this_iter->iteration = iteration;
-	    this_iter->done = FALSE;
-	    this_iter->really_done = FALSE;
-	    this_iter->empty_iteration = FALSE;
-	    this_iter->next = NULL;
-	    this_iter->prev = NULL;
-	    if (nesting_depth == 0) {
-		/* first "for" statement: this will be the listhead */
-		iter = this_iter;
-	    }
-	    else {
-		/* not the first "for" statement: attach the newly created node to the end of the list */
-		iter->prev->next = this_iter;  /* iter->prev points to the last node of the list */
-		this_iter->prev = iter->prev;
-	    }
-	    iter->prev = this_iter; /* a shortcut: making the list circular */
 
-	    /* if one iteration in the chain is empty, the whole chain of iterations is empty, too */
-	    if (!iter->empty_iteration) 
-		iter->empty_iteration = empty_iteration;
+	this_iter = gp_alloc(sizeof(t_iterator), "iteration linked list");
+	this_iter->original_udv_value = original_udv_value;
+	this_iter->iteration_udv = iteration_udv; 
+	this_iter->iteration_string = iteration_string;
+	this_iter->iteration_start = iteration_start;
+	this_iter->iteration_end = iteration_end;
+	this_iter->iteration_increment = iteration_increment;
+	this_iter->iteration_current = iteration_current;
+	this_iter->iteration = iteration;
+	this_iter->start_at = iteration_start_at;
+	this_iter->end_at = iteration_end_at;
+	this_iter->next = NULL;
 
-	    nesting_depth++;
+	if (nesting_depth == 0) {
+	    /* first "for" statement: this will be the listhead */
+	    iter = this_iter;
+	} else {
+	    /* nested "for": attach newly created node to the end of the list */
+	    prev->next = this_iter;
 	}
+	prev = this_iter;
+
+	/* If some depth of a nested iteration evaluates to an empty range, the
+	 * evaluated limits of depths below it are moot (and possibly invalid).
+	 * This flag tells us to skip their evaluation to avoid irrelevant errors.
+	 */
+	if (no_iteration(this_iter)) {
+	    no_parent = TRUE;
+	    FPRINTF((stderr,"iteration at level %d is moot\n", nesting_depth));
+	}
+
+	nesting_depth++;
     }
 
     return iter;
 }
 
-/* Set up next iteration.
- * Return TRUE if there is one, FALSE if we're done
+/*
+ * Reevaluate the iteration limits
+ * (in case they are functions whose parameters have taken 
+ * on a new value)
+ */
+static void
+reevaluate_iteration_limits(t_iterator *iter)
+{
+    if (iter->start_at) {
+	struct value v;
+	evaluate_at(iter->start_at, &v);
+	if (iter->iteration_string) {
+	    /* unnecessary if iteration string is a constant */
+	    free(iter->iteration_string);
+	    if (v.type != STRING)
+		int_error(NO_CARET, "corrupt iteration string");
+	    iter->iteration_string = v.v.string_val;
+	    iter->iteration_start = 1;
+	    iter->iteration_end = gp_words(iter->iteration_string);
+	} else {
+	    iter->iteration_start = real(&v);
+	}
+    }
+    if (iter->end_at) {
+	struct value v;
+	evaluate_at(iter->end_at, &v);
+	iter->iteration_end = real(&v);
+    }
+}
+
+/*
+ * Reset iteration at this level to start value.
+ * Any iteration levels underneath are reset also.
+ */
+static void
+reset_iteration(t_iterator *iter)
+{
+    if (!iter)
+	return;
+
+    reevaluate_iteration_limits(iter);
+    iter->iteration = -1;
+    iter->iteration_current = iter->iteration_start;
+    if (iter->iteration_string) {
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	Gstring(&(iter->iteration_udv->udv_value), 
+		gp_word(iter->iteration_string, iter->iteration_current));
+    } else {
+	/* This traps fatal user error of reassigning iteration variable to a string */
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	Ginteger(&(iter->iteration_udv->udv_value), iter->iteration_current);	
+    }
+    reset_iteration(iter->next);
+}
+
+/*
+ * Increment the iteration position recursively.
+ * returns TRUE if the iteration is still in range
+ * returns FALSE if the incement put it past the end limit
  */
 TBOOLEAN
 next_iteration(t_iterator *iter)
 {
-    t_iterator *this_iter;
-    TBOOLEAN condition = FALSE;
-    
-    if (!iter || iter->empty_iteration)
+    /* Once it goes out of range it will stay that way until reset */
+    if (!iter || no_iteration(iter))
 	return FALSE;
 
-    /* Support for nested iteration:
-     * we start with the innermost loop. */
-    this_iter = iter->prev; /* linked to the last element of the list */
-    
-    if (!this_iter)
-	return FALSE;
-    
-    while (!iter->really_done && this_iter != iter && this_iter->done) {
-	this_iter->iteration_current = this_iter->iteration_start;
-	this_iter->done = FALSE;
-	if (this_iter->iteration_string) {
-	    free(this_iter->iteration_udv->udv_value.v.string_val);
-	    this_iter->iteration_udv->udv_value.v.string_val
-	         = gp_word(this_iter->iteration_string, this_iter->iteration_current);
-    } else
-	    this_iter->iteration_udv->udv_value.v.int_val = this_iter->iteration_current;	
-	
-	this_iter = this_iter->prev;
+    /* Give sub-iterations a chance to advance */
+    if (next_iteration(iter->next)) {
+	if (iter->iteration < 0)
+	    iter->iteration = 0;
+	return TRUE;
+    }
+
+    /* Increment at this level */
+    if (iter->iteration < 0) {
+	/* Just reset, haven't used start value yet */
+	iter->iteration = 0;
+	if (!empty_iteration(iter))
+	    return TRUE;
+    } else {
+	iter->iteration++;
+	iter->iteration_current += iter->iteration_increment;
+    }
+    if (iter->iteration_string) {
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	Gstring(&(iter->iteration_udv->udv_value), 
+		gp_word(iter->iteration_string, iter->iteration_current));
+    } else {
+	/* This traps fatal user error of reassigning iteration variable to a string */
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	Ginteger(&(iter->iteration_udv->udv_value), iter->iteration_current);	
     }
    
-    if (!this_iter->iteration_udv) {
-	this_iter->iteration = 0;
+    /* If this runs off the end, leave the value out-of-range and return FALSE */ 
+    if (iter->iteration_increment > 0 &&  iter->iteration_end - iter->iteration_current < 0)
 	return FALSE;
-    }
-    iter->iteration++;
-    /* don't increment if we're at the last iteration */
-    if (!iter->really_done)
-	this_iter->iteration_current += this_iter->iteration_increment;
-    if (this_iter->iteration_string) {
-	free(this_iter->iteration_udv->udv_value.v.string_val);
-	this_iter->iteration_udv->udv_value.v.string_val
-	    = gp_word(this_iter->iteration_string, this_iter->iteration_current);
-    } else
-	this_iter->iteration_udv->udv_value.v.int_val = this_iter->iteration_current;
-    
-    /* no need to check for increment <> 0 here, 
-     * because that case was already caught in check_for_iteration */
-    this_iter->done = 
-	!((this_iter->iteration_end - this_iter->iteration_current - this_iter->iteration_increment)
-	   * this_iter->iteration_increment >= 0);
-    
-    /* We return false only if we're, erm, really done */
-    this_iter = iter;
-    while (this_iter) {
-	condition = condition || (!this_iter->done);
-	this_iter = this_iter->next;
-    }
-    if (!condition) {
-	if (!iter->really_done) {
-	    iter->really_done = TRUE;
-	    condition = TRUE;
-	} else 
-	    condition = FALSE;
-    }
-    return condition;
+    if (iter->iteration_increment < 0 &&  iter->iteration_end - iter->iteration_current > 0)
+	return FALSE;
+
+    if (iter->next == NULL)
+	return TRUE;
+
+    /* Reset sub-iterations, if any */
+    reset_iteration(iter->next);
+
+    /* Go back to top or call self recursively */
+    return next_iteration(iter);
 }
 
+/*
+ * Returns TRUE if
+ * - this really is an iteration and
+ * - the top level iteration covers no usable range
+ */
+static TBOOLEAN
+no_iteration(t_iterator *iter)
+{
+    if (!iter)
+	return FALSE;
+
+    if ((iter->iteration_end > iter->iteration_start && iter->iteration_increment < 0)
+    ||  (iter->iteration_end < iter->iteration_start && iter->iteration_increment > 0)) {
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
+/*
+ * Recursive test that no empty iteration exists in a nested set of iterations
+ */
 TBOOLEAN
 empty_iteration(t_iterator *iter)
 {
     if (!iter)
 	return FALSE;
-
-    /* FIXME: shouldn't we warn or error out instead? */
-
-    return iter->empty_iteration;
+    else if (no_iteration(iter))
+	return TRUE;
+    else
+	return no_iteration(iter->next);
 }
 
 t_iterator *
@@ -1210,9 +1569,54 @@ cleanup_iteration(t_iterator *iter)
 {
     while (iter) {
 	t_iterator *next = iter->next;
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	iter->iteration_udv->udv_value = iter->original_udv_value;
 	free(iter->iteration_string);
+	free_at(iter->start_at);
+	free_at(iter->end_at);
 	free(iter);
 	iter = next;
     }
     return NULL;
 }
+
+TBOOLEAN
+forever_iteration(t_iterator *iter)
+{
+    if (!iter)
+	return FALSE;
+    else
+	return (iter->iteration_end == INT_MAX);
+}
+
+/* The column() function requires special handling because
+ * - It has side effects if reference to a column entry
+ *   requires matching it to the column header string.
+ * - These side effects must be handled at the time the
+ *   expression is parsed rather than when it it evaluated.
+ */
+static void
+set_up_columnheader_parsing( struct at_entry *previous )
+{
+    /* column("string") means we expect the first row of */
+    /* a data file to contain headers rather than data.  */
+    if (previous->index == PUSHC &&  previous->arg.v_arg.type == STRING)
+	parse_1st_row_as_headers = TRUE;
+
+    /* This allows plot ... using (column(<const>)) title columnhead */
+    if (previous->index == PUSHC && previous->arg.v_arg.type == INTGR) {
+	if (at_highest_column_used < previous->arg.v_arg.v.int_val)
+	    at_highest_column_used = previous->arg.v_arg.v.int_val;
+    }
+    
+    /* This attempts to catch plot ... using (column(<variable>)) */
+    if (previous->index == PUSH) {
+	udvt_entry *u = previous->arg.udv_arg;
+	if (u->udv_value.type == INTGR) {
+	    if (at_highest_column_used < u->udv_value.v.int_val)
+		at_highest_column_used = u->udv_value.v.int_val;
+	}
+    }
+
+    /* NOTE: There is no way to handle ... using (column(<general expression>)) */
+}
