@@ -1,5 +1,5 @@
 /*
- * $Id: plot.h,v 1.55.2.1 2017/05/22 12:31:01 markisch Exp $
+ * $Id: plot.h,v 1.56 2017/05/20 16:43:19 markisch Exp $
  */
 
 /* GNUPLOT - plot.h */
@@ -74,7 +74,7 @@ void bail_to_command_line __PROTO((void));
 void init_constants __PROTO((void));
 void init_session __PROTO((void));
 
-#if defined(_Windows)
+#if defined(_WIN32)
 int gnu_main __PROTO((int argc, char **argv));
 #endif
 
