@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.104.2.2 2016/12/15 20:34:25 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.117.2.1 2017/10/13 19:51:45 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -19,6 +19,7 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.104.2.2 2016/12/15 20:34:25
 #endif
 #include "pm3d.h"
 #include "alloc.h"
+#include "getcolor.h"		/* for rgb1_from_gray */
 #include "graphics.h"
 #include "hidden3d.h"		/* p_vertex & map3d_xyz() */
 #include "plot2d.h"
@@ -34,6 +35,7 @@ struct lp_style_type default_pm3d_border = DEFAULT_LP_STYLE_TYPE;
 
 /* Used by routine filled_quadrangle() in color.c */
 struct lp_style_type pm3d_border_lp;
+TBOOLEAN track_pm3d_quadrangles;
 
 /*
   Global options for pm3d algorithm (to be accessed by set / show).
@@ -52,6 +54,8 @@ pm3d_struct pm3d = {
     DEFAULT_LP_STYLE_TYPE	/* for the border */
 };
 
+lighting_model pm3d_shade;
+
 typedef struct {
     double gray;
     double z; /* maximal z value after rotation to graph coordinate system */
@@ -62,6 +66,8 @@ typedef struct {
     t_colorspec *border_color;	/* Only used by depthorder processing */
 } quadrangle;
 
+#define PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY -12345
+
 static int allocated_quadrangles = 0;
 static int current_quadrangle = 0;
 static quadrangle* quadrangles = (quadrangle*)0;
@@ -75,7 +81,10 @@ static double rms4 __PROTO((double, double, double, double));
 static void pm3d_plot __PROTO((struct surface_points *, int));
 static void pm3d_option_at_error __PROTO((void));
 static void pm3d_rearrange_part __PROTO((struct iso_curve *, const int, struct iso_curve ***, int *));
+static int apply_lighting_model __PROTO(( struct coordinate *, struct coordinate *, struct coordinate *, struct coordinate *, double gray ));
+
 static TBOOLEAN color_from_rgbvar = FALSE;
+static double light[3];
 
 /*
  * Utility routines.
@@ -164,21 +173,24 @@ rms4 (double x1, double x2, double x3, double x4)
  * Rescale z to cb values. Nothing to do if both z and cb are linear or log of the
  * same base, other it has to un-log z and subsequently log it again.
  */
+#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
+    /* z2cb is a no-op */
+#else
 double
-z2cb(double z)
+z2cb_with_logs(double z)
 {
     if (!Z_AXIS.log && !CB_AXIS.log) /* both are linear */
 	return z;
     if (Z_AXIS.log && !CB_AXIS.log) /* log z, linear cb */
-	return exp(z * Z_AXIS.log_base); /* unlog(z) */
+	return axis_undo_log((&Z_AXIS), z);
     if (!Z_AXIS.log && CB_AXIS.log) /* linear z, log cb */
-	return (z<=0) ? CB_AXIS.min : (log(z) / CB_AXIS.log_base);
+	return (z<=0) ? CB_AXIS.min : axis_do_log((&CB_AXIS), z);
     /* both are log */
     if (Z_AXIS.base==CB_AXIS.base) /* can we compare double numbers like that? */
 	return z;
     return z * Z_AXIS.log_base / CB_AXIS.log_base; /* log_cb(unlog_z(z)) */
 }
-
+#endif
 
 /*
  * Rescale cb (color) value into the interval of grays [0,1], taking care
@@ -188,12 +200,19 @@ z2cb(double z)
 double
 cb2gray(double cb)
 {
-    if (cb <= CB_AXIS.min)
+    AXIS *cbaxis = &CB_AXIS;
+
+    if (cb <= cbaxis->min)
 	return (sm_palette.positive == SMPAL_POSITIVE) ? 0 : 1;
-    if (cb >= CB_AXIS.max)
+    if (cb >= cbaxis->max)
 	return (sm_palette.positive == SMPAL_POSITIVE) ? 1 : 0;
-    cb = (cb - CB_AXIS.min)
-      / (CB_AXIS.max - CB_AXIS.min);
+
+    if (nonlinear(cbaxis)) {
+	cbaxis = cbaxis->linked_to_primary;
+	cb = eval_link_function(cbaxis, cb);
+    }
+
+    cb = (cb - cbaxis->min) / (cbaxis->max - cbaxis->min);
     return (sm_palette.positive == SMPAL_POSITIVE) ? cb : 1-cb;
 }
 
@@ -354,21 +373,19 @@ static int compare_quadrangles(const void* v1, const void* v2)
 
 void pm3d_depth_queue_clear(void)
 {
-    if (pm3d.direction != PM3D_DEPTH)
-	return;
-
-    if (quadrangles)
-	free(quadrangles);
-    quadrangles = (quadrangle*)0;
+    free(quadrangles);
+    quadrangles = NULL;
     allocated_quadrangles = 0;
     current_quadrangle = 0;
 }
 
 void pm3d_depth_queue_flush(void)
 {
-    if (pm3d.direction != PM3D_DEPTH)
+    if (pm3d.direction != PM3D_DEPTH && !track_pm3d_quadrangles)
 	return;
 
+    term->layer(TERM_LAYER_BEGIN_PM3D_FLUSH);
+
     if (current_quadrangle > 0 && quadrangles) {
 
 	quadrangle* qp;
@@ -410,14 +427,14 @@ void pm3d_depth_queue_flush(void)
 
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
-	    if (color_from_rgbvar)
-		set_rgbcolor_var(qp->gray);
+	    /* set the color */
+	    if (qp->gray == PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY)
+		apply_pm3dcolor(qp->border_color);
+	    else if (color_from_rgbvar || pm3d_shade.strength > 0)
+		set_rgbcolor_var((unsigned int)qp->gray);
 	    else
 		set_color(qp->gray);
-#if (0)	/* FIXME: It used to do this, but I can't see when it would make sense */
-	    if (pm3d.border.l_type < 0)
-		pm3d_border_lp.pm3d_color = *(qp->border_color);
-#endif
+
 #ifdef EXTENDED_COLOR_SPECS
 	    ifilled_quadrangle(qp->icorners);
 #else
@@ -427,13 +444,12 @@ void pm3d_depth_queue_flush(void)
     }
 
     pm3d_depth_queue_clear();
+
+    term->layer(TERM_LAYER_END_PM3D_FLUSH);
 }
 
 /*
  * Now the implementation of the pm3d (s)plotting mode
- *
- * Note: the input parameter at_which_z is char, but an old HP cc requires
- * ANSI C K&R routines with int only.
  */
 static void
 pm3d_plot(struct surface_points *this_plot, int at_which_z)
@@ -474,6 +490,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     if (!term->filled_polygon)
 	return;
 
+    /* for pm3dCompress.awk and pm3dConvertToImage.awk */
+    if (pm3d.direction != PM3D_DEPTH)
+	term->layer(TERM_LAYER_BEGIN_PM3D_MAP);
+
     switch (at_which_z) {
     case PM3D_AT_BASE:
 	corners[0].z = corners[1].z = corners[2].z = corners[3].z = base_z;
@@ -516,6 +536,9 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 	}
 
+	if (max_scan_pts == 0 || max_scans == 0)
+	    int_error(NO_CARET, "all scans empty");
+
 	if (interp_i <= 0) {
 	    ii = (interp_i == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_i;
 	    interp_i = floor(ii / max_scan_pts) + 1;
@@ -689,6 +712,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 
 	    if ((interp_i <= 1 && interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
+
 #ifdef EXTENDED_COLOR_SPECS
 	      if ((term->flags & TERM_EXTENDED_COLOR) == 0)
 #endif
@@ -697,7 +721,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		   (note: log scale is already included). The average is calculated here
 		   if there is no interpolation (including the "pm3d depthorder" option),
 		   otherwise it is done for each interpolated quadrangle later.
-		   I always wonder what is faster: d*0.25 or d/4? Someone knows? -- 0.25 (joze) */
+		 */
 		if (color_from_column) {
 		    /* color is set in plot3d.c:get_3ddata() */
 		    cb1 = pointsA[i].CRD_COLOR;
@@ -754,20 +778,31 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		if (isnan(avgC))
 		    continue;
 
-		if (color_from_rgbvar) /* we were given an explicit color */
+		if (color_from_rgbvar) /* we were given an RGB color */
 			gray = avgC;
 		else /* transform z value to gray, i.e. to interval [0,1] */
 			gray = cb2gray(avgC);
 
-		/* print the quadrangle with the given color */
-		FPRINTF((stderr, "averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
-		       avgC, gray, pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
-		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y));
+		/* apply lighting model */
+		if (pm3d_shade.strength > 0) {
+		    if (at_which_z == PM3D_AT_SURFACE)
+			gray = apply_lighting_model( &pointsA[i], &pointsA[i1],
+					&pointsB[ii], &pointsB[ii1], gray);
+		    /* Don't apply lighting model to TOP/BOTTOM projections  */
+		    /* but convert from floating point 0<gray<1 to RGB color */
+		    /* since that is what would have been returned from the  */
+		    /* lighting code.					     */
+		    else if (!color_from_rgbvar) {
+			rgb255_color temp;
+			rgb255maxcolors_from_gray(gray, &temp);
+			gray = (long)((temp.r << 16) + (temp.g << 8) + (temp.b));
+		    }
+		}
 
 		/* set the color */
 		if (pm3d.direction != PM3D_DEPTH) {
-		    if (color_from_rgbvar)
-			set_rgbcolor_var(gray);
+		    if (color_from_rgbvar || pm3d_shade.strength > 0)
+			set_rgbcolor_var((unsigned int)gray);
 		    else
 			set_color(gray);
 		}
@@ -784,9 +819,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    corners[3].y = pointsA[i1].y;
 
 	    if (interp_i > 1 || interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
-		/* always supply the z value if
-		 * EXTENDED_COLOR_SPECS is defined
-		 */
 		corners[0].z = pointsA[i].z;
 		corners[1].z = pointsB[ii].z;
 		corners[2].z = pointsB[ii1].z;
@@ -906,18 +938,19 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    corners[2].x, corners[2].y, corners[3].x, corners[3].y));
 
 			/* If the colors are given separately, we already loaded them above */
-			if (!color_from_column) {
-			    cb1 = z2cb(corners[0].z);
-			    cb2 = z2cb(corners[1].z);
-			    cb3 = z2cb(corners[2].z);
-			    cb4 = z2cb(corners[3].z);
-			} else {
+			if (color_from_column) {
 			    cb1 = corners[0].c;
 			    cb2 = corners[1].c;
 			    cb3 = corners[2].c;
 			    cb4 = corners[3].c;
+			} else {
+			    cb1 = z2cb(corners[0].z);
+			    cb2 = z2cb(corners[1].z);
+			    cb3 = z2cb(corners[2].z);
+			    cb4 = z2cb(corners[3].z);
 			}
 			switch (pm3d.which_corner_color) {
+			    default:
 			    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
 			    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_HARMEAN: avgC = harmean4(cb1, cb2, cb3, cb4); break;
@@ -929,7 +962,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
 			    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
 			    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
-			    default: int_error(NO_CARET, "cannot be here"); avgC = 0;
 			}
 
 			if (color_from_rgbvar) /* we were given an explicit color */
@@ -937,6 +969,31 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			else /* transform z value to gray, i.e. to interval [0,1] */
 				gray = cb2gray(avgC);
 
+			/* apply lighting model */
+			if (pm3d_shade.strength > 0) {
+			    /* FIXME: coordinate->quadrangle->coordinate seems crazy */
+			    coordinate corcorners[4];
+			    int i;
+			    for (i=0; i<4; i++) {
+				corcorners[i].x = corners[i].x;
+				corcorners[i].y = corners[i].y;
+				corcorners[i].z = corners[i].z;
+			    }
+
+			    if (at_which_z == PM3D_AT_SURFACE)
+				gray = apply_lighting_model( &corcorners[0], &corcorners[1],
+						&corcorners[2], &corcorners[3], gray);
+			    /* Don't apply lighting model to TOP/BOTTOM projections  */
+			    /* but convert from floating point 0<gray<1 to RGB color */
+			    /* since that is what would have been returned from the  */
+			    /* lighting code.					     */
+			    else if (!color_from_rgbvar) {
+				rgb255_color temp;
+				rgb255maxcolors_from_gray(gray, &temp);
+				gray = (long)((temp.r << 16) + (temp.g << 8) + (temp.b));
+			    }
+			}
+	
 			if (pm3d.direction == PM3D_DEPTH) {
 			    /* copy quadrangle */
 			    quadrangle* qp = quadrangles + current_quadrangle;
@@ -945,8 +1002,8 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    qp->border_color = &this_plot->lp_properties.pm3d_color;
 			    current_quadrangle++;
 			} else {
-			    if (color_from_rgbvar)
-				set_rgbcolor_var(gray);
+			    if (pm3d_shade.strength > 0 || color_from_rgbvar)
+				set_rgbcolor_var((unsigned int)gray);
 			    else
 				set_color(gray);
 			    if (at_which_z == PM3D_AT_BASE)
@@ -980,8 +1037,13 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     }
     /* free memory allocated by scan_array */
     free(scan_array);
+
+    /* for pm3dCompress.awk and pm3dConvertToImage.awk */
+    if (pm3d.direction != PM3D_DEPTH)
+	term->layer(TERM_LAYER_END_PM3D_MAP);
 }				/* end of pm3d splotting mode */
 
+
 #ifdef PM3D_CONTOURS
 /*
  *  Now the implementation of the filled color contour plot
@@ -1049,6 +1111,10 @@ pm3d_reset()
     pm3d.interp_i = 1;
     pm3d.interp_j = 1;
     pm3d.border.l_type = LT_NODRAW;
+
+    pm3d_shade.strength = 0.0;
+    pm3d_shade.spec = 0.0;
+    pm3d_shade.fixed = TRUE;
 }
 
 
@@ -1066,9 +1132,12 @@ pm3d_draw_one(struct surface_points *plot)
     if (!where[0])
 	return;
 
-    /* for pm3dCompress.awk */
-    if (pm3d.direction != PM3D_DEPTH)
-	term->layer(TERM_LAYER_BEGIN_PM3D_MAP);
+    /* Initialize lighting model */
+    if (pm3d_shade.strength > 0) {
+	light[0] = cos(-DEG2RAD*pm3d_shade.rot_x)*cos(-(DEG2RAD*pm3d_shade.rot_z+90));
+	light[2] = cos(-DEG2RAD*pm3d_shade.rot_x)*sin(-(DEG2RAD*pm3d_shade.rot_z+90));
+	light[1] = sin(-DEG2RAD*pm3d_shade.rot_x);
+    }
 
     for (; where[i]; i++) {
 	pm3d_plot(plot, where[i]);
@@ -1089,13 +1158,28 @@ pm3d_draw_one(struct surface_points *plot)
 	    filled_color_contour_plot(plot, CONTOUR_BASE);
     }
 #endif
-
-    /* for pm3dCompress.awk */
-    if (pm3d.direction != PM3D_DEPTH)
-	term->layer(TERM_LAYER_END_PM3D_MAP);
 }
 
 
+/*
+ * Add one pm3d quadrangle to the mix.
+ * Called by zerrorfill().
+ */
+void
+pm3d_add_quadrangle(struct surface_points *plot, gpdPoint corners[4])
+{
+    quadrangle *q;
+
+    if (allocated_quadrangles < current_quadrangle + plot->iso_crvs->p_count) {
+	allocated_quadrangles += 2 * plot->iso_crvs->p_count;
+	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
+    }
+    q = quadrangles + current_quadrangle++;
+    memcpy(q->corners, corners, 4*sizeof(gpdPoint));
+    q->border_color = &plot->fill_properties.border_color;
+    q->gray = PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY;
+}
+
 /* Display an error message for the routine get_pm3d_at_option() below.
  */
 
@@ -1213,11 +1297,12 @@ set_plot_with_palette(int plot_num, int plot_mode)
     if (plot_mode == MODE_SPLOT)
 	if (TC_USES_PALETTE(axis_array[FIRST_Z_AXIS].label.textcolor.type)) return;
     if (TC_USES_PALETTE(axis_array[COLOR_AXIS].label.textcolor.type)) return;
-
+#ifdef EAM_OBJECTS
     for (this_object = first_object; this_object != NULL; this_object = this_object->next) {
 	if (TC_USES_PALETTE(this_object->lp_properties.pm3d_color.type))
 	    return;
     }
+#endif
     
 #undef TC_USES_PALETTE
 
@@ -1237,3 +1322,108 @@ is_plot_with_colorbox()
     return plot_has_palette && (color_box.where != SMCOLOR_BOX_NO);
 }
 
+/*
+ * Adjust current RGB color based on pm3d lighting model.
+ */
+int
+apply_lighting_model( struct coordinate *v0, struct coordinate *v1, 
+		struct coordinate *v2, struct coordinate *v3,
+		double gray )
+{
+    double normal[3];
+    double normal1[3];
+    double reflect[3];
+    double t;
+    double phi;
+    double psi;
+    int rgb;
+    rgb_color color;
+    double r, g, b, tmp_r, tmp_g, tmp_b;
+    double dot_prod, shade_fact, spec_fact;
+
+    if (color_from_rgbvar) {
+	rgb = gray;
+	r = (double)((rgb >> 16) & 0xFF) / 255.;
+	g = (double)((rgb >>  8) & 0xFF) / 255.;
+	b = (double)((rgb      ) & 0xFF) / 255.;
+    } else {
+	rgb1_from_gray(gray, &color);
+	r = color.r;
+	g = color.g;
+	b = color.b;	   
+    }
+
+    psi = -DEG2RAD*(surface_rot_z);
+    phi = -DEG2RAD*(surface_rot_x);
+
+    normal[0] = (v1->y-v0->y)*(v2->z-v0->z)*yscale3d*zscale3d
+	      - (v1->z-v0->z)*(v2->y-v0->y)*yscale3d*zscale3d;
+    normal[1] = (v1->z-v0->z)*(v2->x-v0->x)*xscale3d*zscale3d
+	      - (v1->x-v0->x)*(v2->z-v0->z)*xscale3d*zscale3d;
+    normal[2] = (v1->x-v0->x)*(v2->y-v0->y)*xscale3d*yscale3d
+	      - (v1->y-v0->y)*(v2->x-v0->x)*xscale3d*yscale3d ;
+
+    t = sqrt( normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2] );
+
+    normal[0] /= t;    
+    normal[1] /= t;    
+    normal[2] /= t;    
+
+    /* Correct for the view angle so that the illumination is "fixed" with */
+    /* respect to the viewer rather than rotating with the surface.        */
+    if (pm3d_shade.fixed) {
+	normal1[0] =  cos(psi)*normal[0] -  sin(psi)*normal[1] + 0*normal[2];
+	normal1[1] =  sin(psi)*normal[0] +  cos(psi)*normal[1] + 0*normal[2];
+	normal1[2] =  0*normal[0] +                0*normal[1] + 1*normal[2];
+	
+	normal[0] =  1*normal1[0] +         0*normal1[1] +         0*normal1[2];
+	normal[1] =  0*normal1[0] +  cos(phi)*normal1[1] -  sin(phi)*normal1[2];
+	normal[2] =  0*normal1[0] +  sin(phi)*normal1[1] +  cos(phi)*normal1[2];
+    }
+
+    if (normal[2] < 0.0) {
+	normal[0] *= -1.0;
+	normal[1] *= -1.0;
+	normal[2] *= -1.0;
+    }
+
+    dot_prod = normal[0]*light[0] + normal[1]*light[1] + normal[2]*light[2];
+    shade_fact = (dot_prod < 0) ? -dot_prod : 0;
+
+    tmp_r = r*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+    tmp_g = g*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+    tmp_b = b*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+
+    /* Specular highlighting */ 
+    if (pm3d_shade.spec > 0.0) {
+
+	reflect[0] = -light[0]+2*dot_prod*normal[0];
+	reflect[1] = -light[1]+2*dot_prod*normal[1];
+	reflect[2] = -light[2]+2*dot_prod*normal[2];
+	t = sqrt( reflect[0]*reflect[0] + reflect[1]*reflect[1] + reflect[2]*reflect[2] );
+	reflect[0] /= t;
+	reflect[1] /= t;
+	reflect[2] /= t;
+	
+	dot_prod = -reflect[2];
+	if (dot_prod < 0.0)
+	    dot_prod = 0;
+
+	/* old-style Phong equation; no need for bells or whistles */
+	spec_fact = pow(dot_prod, pm3d_shade.Phong);
+
+	tmp_r += pm3d_shade.spec*spec_fact;
+	tmp_g += pm3d_shade.spec*spec_fact;
+	tmp_b += pm3d_shade.spec*spec_fact;
+    }
+
+    tmp_r = clip_to_01(tmp_r);
+    tmp_g = clip_to_01(tmp_g);
+    tmp_b = clip_to_01(tmp_b);
+
+    rgb = ((unsigned char)((tmp_r)*255.) << 16)
+	+ ((unsigned char)((tmp_g)*255.) <<  8)
+	+ ((unsigned char)((tmp_b)*255.));
+
+    return rgb;
+}
