@@ -1,5 +1,5 @@
 /*
- * $Id: pm3d.h,v 1.28 2010/10/06 23:20:50 sfeam Exp $
+ * $Id: pm3d.h,v 1.36 2016/11/29 22:56:39 sfeam Exp $
  */
 
 /* GNUPLOT - pm3d.h */
@@ -26,7 +26,8 @@
 #ifndef PM3D_H
 #define PM3D_H
 
-#include "graph3d.h" /* struct surface_points */
+#include "axis.h"	/* only for NONLINEAR_AXES */
+#include "graph3d.h"	/* struct surface_points */
 
 
 
@@ -56,10 +57,12 @@
   direction of taking the scans: forward = as the scans are stored in the
   file; backward = opposite direction, i.e. like from the end of the file
 */
-#define PM3D_SCANS_AUTOMATIC  'a'
-#define PM3D_SCANS_FORWARD    'f'
-#define PM3D_SCANS_BACKWARD   'b'
-#define PM3D_DEPTH            'd'
+typedef enum {
+    PM3D_SCANS_AUTOMATIC,
+    PM3D_SCANS_FORWARD,
+    PM3D_SCANS_BACKWARD,
+    PM3D_DEPTH
+} pm3d_scandir;
 
 /*
   clipping method:
@@ -87,12 +90,15 @@ typedef enum {
     PM3D_WHICHCORNER_C2 = 1, 	/* corner 2: first scan, second point  */
     PM3D_WHICHCORNER_C3 = 2, 	/* corner 3: second scan, first point  */
     PM3D_WHICHCORNER_C4 = 3,	/* corner 4: second scan, second point */
-    /* now in any order */
+    /* the rest can be in any order */
     PM3D_WHICHCORNER_MEAN    = 4, /* average z-value from all 4 corners */
     PM3D_WHICHCORNER_GEOMEAN = 5, /* geometrical mean of 4 corners */
-    PM3D_WHICHCORNER_MEDIAN  = 6, /* median of 4 corners */
-    PM3D_WHICHCORNER_MIN     = 7, /* minimum of 4 corners */
-    PM3D_WHICHCORNER_MAX     = 8  /* maximum of 4 corners */
+    PM3D_WHICHCORNER_HARMEAN = 6, /* harmonic mean of 4 corners */
+    PM3D_WHICHCORNER_MEDIAN  = 7, /* median of 4 corners */
+    PM3D_WHICHCORNER_RMS     = 8, /* root mean square of 4 corners*/
+    PM3D_WHICHCORNER_MIN     = 9, /* minimum of 4 corners */
+    PM3D_WHICHCORNER_MAX     = 10,/* maximum of 4 corners */
+    PM3D_COLOR_BY_NORMAL     = 11 /* derive color from surface normal (not currently used) */
 } PM3D_WHICH_CORNERS2COLOR;
 
 /*
@@ -103,10 +109,8 @@ typedef struct {
   char where[7];	/* base, top, surface */
   char flush;   	/* left, right, center */
   char ftriangles;   	/* 0/1 (don't) draw flushing triangles */
-  char direction;	/* forward, backward */
   char clip;		/* 1in, 4in */
-  int hidden3d_tag;     /* if this is > 0, pm3d hidden lines are drawn with
-			   this linestyle (which must naturally present). */
+  pm3d_scandir direction;
   PM3D_IMPL_MODE implicit;
 			/* 1: [default] draw ALL surfaces with pm3d
 			   0: only surfaces specified with 'with pm3d' */
@@ -114,12 +118,36 @@ typedef struct {
 			/* default: average color from all 4 points */
   int interp_i;		/* # of interpolation steps along scanline */
   int interp_j;		/* # of interpolation steps between scanlines */
+  lp_style_type border;	/* LT_NODRAW to disable.  From `set pm3d border <linespec> */
 } pm3d_struct;
 
-
 extern pm3d_struct pm3d;
-extern struct lp_style_type pm3d_border_lp;
 
+typedef struct lighting_model {
+  double strength;	/* 0 = no lighting model; 1 = full shading */
+  double spec;		/* specular component 0-1 */
+  double ambient;	/* ambient component 0-1 */
+  double Phong;		/* Phong exponent */
+  int rot_z;		/* illumination angle */
+  int rot_x;		/* illumination angle */
+  TBOOLEAN fixed;	/* TRUE means the light does not rotate */
+} lighting_model;
+
+extern lighting_model pm3d_shade;
+
+/* Used to initialize `set pm3d border` */
+extern struct lp_style_type default_pm3d_border;
+
+/* Used by routine filled_quadrangle() in color.c */
+extern struct lp_style_type pm3d_border_lp;
+extern TBOOLEAN track_pm3d_quadrangles;
+
+#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
+#   define z2cb(z) (z)
+#else
+    /* The original routine, with log/unlog dance steps */
+#   define z2cb(z) z2cb_with_logs(z)
+#endif
 
 
 /****
@@ -131,7 +159,8 @@ void pm3d_depth_queue_clear __PROTO((void));
 void pm3d_depth_queue_flush __PROTO((void));
 void pm3d_reset __PROTO((void));
 void pm3d_draw_one __PROTO((struct surface_points* plots));
-double z2cb __PROTO((double z));
+void pm3d_add_quadrangle __PROTO((struct surface_points* plot, gpdPoint *corners));
+double z2cb_with_logs __PROTO((double z));
 double cb2gray __PROTO((double cb));
 void
 pm3d_rearrange_scan_array __PROTO((struct surface_points* this_plot,
