@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util3d.c,v 1.36 2009/01/07 22:55:42 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util3d.c,v 1.55 2017/05/05 06:10:31 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util3d.c */
@@ -52,13 +52,24 @@ static char *RCSid() { return RCSid("$Id: util3d.c,v 1.36 2009/01/07 22:55:42 sf
 #include "pm3d.h"
 #include "term_api.h"
 
-/* HBB 990826: all that stuff referenced from other modules is now
- * exported in graph3d.h, instead of being listed here */
+#define AXIS_ACTUAL_MIN(axis) GPMIN(axis_array[axis].max, axis_array[axis].min)
+#define AXIS_ACTUAL_MAX(axis) GPMAX(axis_array[axis].max, axis_array[axis].min)
 
 /* Prototypes for local functions */
 static void mat_unit __PROTO((transform_matrix mat));
 static GP_INLINE void draw3d_point_unconditional __PROTO((p_vertex, struct lp_style_type *));
 
+#ifdef NONLINEAR_AXES
+static double map_x3d __PROTO((double));
+static double map_y3d __PROTO((double));
+static double map_z3d __PROTO((double));
+#else
+/* Function macros to map from user 3D space into normalized -1..1 */
+#define map_x3d(x) ((x-X_AXIS.min)*xscale3d + xcenter3d - 1.0)
+#define map_y3d(y) ((y-Y_AXIS.min)*yscale3d + ycenter3d - 1.0)
+#define map_z3d(z) ((z-floor_z)*zscale3d + zcenter3d - 1.0)
+#endif
+
 static void
 mat_unit(transform_matrix mat)
 {
@@ -72,17 +83,6 @@ mat_unit(transform_matrix mat)
 		mat[i][j] = 0.0;
 }
 
-#if 0 /* HBB 990829: unused --> commented out */
-void
-mat_trans(double tx, double ty, double tz, transform_matrix mat)
-{
-    mat_unit(mat);		/* Make it unit matrix. */
-    mat[3][0] = tx;
-    mat[3][1] = ty;
-    mat[3][2] = tz;
-}
-#endif /* commented out */
-
 void
 mat_scale(double sx, double sy, double sz, transform_matrix mat)
 {
@@ -108,24 +108,6 @@ mat_rot_x(double teta, transform_matrix mat)
     mat[2][2] = cos_teta;
 }
 
-#if 0 /* HBB 990829: unused --> commented out */
-void
-mat_rot_y(double teta, transform_matrix mat)
-{
-    double cos_teta, sin_teta;
-
-    teta *= DEG2RAD;
-    cos_teta = cos(teta);
-    sin_teta = sin(teta);
-
-    mat_unit(mat);		/* Make it unit matrix. */
-    mat[0][0] = cos_teta;
-    mat[0][2] = -sin_teta;
-    mat[2][0] = sin_teta;
-    mat[2][2] = cos_teta;
-}
-#endif /* commented out */
-
 void
 mat_rot_z(double teta, transform_matrix mat)
 {
@@ -173,20 +155,19 @@ mat_mult(
  */
 void
 edge3d_intersect(
-    struct coordinate GPHUGE *points,	/* the points array */
-    int i,				/* line segment from point i-1 to point i */
+    coordinate *p1, coordinate *p2,
     double *ex, double *ey, double *ez)	/* the point where it crosses an edge */
 {
     int count;
-    double ix = points[i - 1].x;
-    double iy = points[i - 1].y;
-    double iz = points[i - 1].z;
-    double ox = points[i].x;
-    double oy = points[i].y;
-    double oz = points[i].z;
+    double ix = p1->x;
+    double iy = p1->y;
+    double iz = p1->z;
+    double ox = p2->x;
+    double oy = p2->y;
+    double oz = p2->z;
     double x, y, z;		/* possible intersection point */
 
-    if (points[i].type == INRANGE) {
+    if (p1->type == INRANGE) {
 	/* swap points around so that ix/ix/iz are INRANGE and ox/oy/oz are OUTRANGE */
 	x = ix;
 	ix = ox;
@@ -256,7 +237,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_Z_AXIS), iz, oz))
 		*ez = AXIS_ACTUAL_MIN(FIRST_Z_AXIS);
 	    else {
-		graph_error("error in edge3d_intersect");
+		int_error(NO_CARET,"error in edge3d_intersect");
 	    }
 
 	    return;
@@ -273,7 +254,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_Y_AXIS), iy, oy))
 		*ey = AXIS_ACTUAL_MIN(FIRST_Y_AXIS);
 	    else {
-		graph_error("error in edge3d_intersect");
+		int_error(NO_CARET,"error in edge3d_intersect");
 	    }
 
 	    return;
@@ -323,7 +304,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_X_AXIS), ix, ox))
 		*ex = AXIS_ACTUAL_MIN(FIRST_X_AXIS);
 	    else {
-		graph_error("error in edge3d_intersect");
+		int_error(NO_CARET,"error in edge3d_intersect");
 	    }
 
 	    return;
@@ -430,18 +411,17 @@ edge3d_intersect(
  */
 TBOOLEAN			/* any intersection? */
 two_edge3d_intersect(
-    struct coordinate GPHUGE *points,	/* the points array */
-    int i,				/* line segment from point i-1 to point i */
+    coordinate *p0, coordinate *p1,
     double *lx, double *ly, double *lz)	/* lx[2], ly[2], lz[2]: points where it crosses edges */
 {
     int count;
     /* global axis_array[FIRST_{X,Y,Z}_AXIS].{min,max} */
-    double ix = points[i - 1].x;
-    double iy = points[i - 1].y;
-    double iz = points[i - 1].z;
-    double ox = points[i].x;
-    double oy = points[i].y;
-    double oz = points[i].z;
+    double ix = p0->x;
+    double iy = p0->y;
+    double iz = p0->z;
+    double ox = p1->x;
+    double oy = p1->y;
+    double oz = p1->z;
     double t[6];
     double swap;
     double x, y, z;		/* possible intersection point */
@@ -922,28 +902,9 @@ map3d_xy_double(
     double x, double y, double z,
     double *xt, double *yt)
 {
-    int i, j;
-    double v[4], res[4],	/* Homogeneous coords. vectors. */
-     w = trans_mat[3][3];
-
-    v[0] = map_x3d(x);		/* Normalize object space to -1..1 */
-    v[1] = map_y3d(y);
-    v[2] = map_z3d(z);
-    v[3] = 1.0;
-
-    for (i = 0; i < 2; i++) {	/* Dont use the third axes (z). */
-	res[i] = trans_mat[3][i];	/* Initiate it with the weight factor */
-	for (j = 0; j < 3; j++)
-	    res[i] += v[j] * trans_mat[j][i];
-    }
-
-    for (i = 0; i < 3; i++)
-	w += v[i] * trans_mat[i][3];
-    if (w == 0)
-	w = 1e-5;
-
-    *xt = ((res[0] * xscaler / w) + xmiddle);
-    *yt = ((res[1] * yscaler / w) + ymiddle);
+    vertex v;
+    map3d_xyz(x, y, z, &v);
+    TERMCOORD(&v, *xt, *yt);
 }
 
 
@@ -955,11 +916,9 @@ draw3d_point_unconditional(p_vertex v, struct lp_style_type *lp)
     unsigned int x, y;
 
     TERMCOORD(v, x, y);
+    /* Jul 2010 EAM - is it safe to overwrite like this? Make a copy instead? */
+    lp->pm3d_color.value = v->real_z;
     term_apply_lp_properties(lp);
-    /* HBB 20010822: implemented "linetype palette" for points, too */
-    if (lp->use_palette) {
-	set_color(cb2gray( z2cb(v->real_z) ));
-    }
     if (!clip_point(x, y))
 	(term->point) (x, y, lp->p_type);
 }
@@ -971,7 +930,7 @@ void
 draw3d_line_unconditional(
     p_vertex v1, p_vertex v2,
     struct lp_style_type *lp,
-    int linetype)
+    t_colorspec color)
 {
     unsigned int x1, y1, x2, y2;
     struct lp_style_type ls = *lp;
@@ -986,34 +945,39 @@ draw3d_line_unconditional(
     TERMCOORD(v1, x1, y1);
     TERMCOORD(v2, x2, y2);
 
-    /* User-specified line styles */
-    if (prefer_line_styles && linetype >= 0)
-	lp_use_properties(&ls, linetype+1);
-
-    /* The usual case of auto-generated line types */
-    else
-	ls.l_type = linetype;
+    /* Replace original color with the one passed in */
+    ls.pm3d_color = color;
 
     /* Color by Z value */
     if (ls.pm3d_color.type == TC_Z)
 	    ls.pm3d_color.value = (v1->real_z + v2->real_z) * 0.5;
 
-    term_apply_lp_properties(&ls);
-    draw_clip_line(x1,y1,x2,y2);
+    /* This interrupts the polyline, messing up dash patterns. */
+    /* Skip it if the caller indicates that the line properties are */
+    /* already set by passing in color.type = TC_DEFAULT */
+    if (color.type != TC_DEFAULT)
+	term_apply_lp_properties(&ls);
+
+    /* Support for hidden3d VECTOR mode with arrowheads */
+    if (lp->p_type == PT_ARROWHEAD)
+	draw_clip_arrow(x1,y1,x2,y2,END_HEAD);
+    else if (lp->p_type == PT_BACKARROW)
+	draw_clip_arrow(x1,y1,x2,y2,BACKHEAD);
+    else
+
+	draw_clip_line(x1,y1,x2,y2);
 }
 
 void
 draw3d_line (p_vertex v1, p_vertex v2, struct lp_style_type *lp)
 {
-#ifndef LITE
     /* hidden3d routine can't work if no surface was drawn at all */
     if (hidden3d && draw_surface) {
 	draw_line_hidden(v1, v2, lp);
 	return;
     }
-#endif
 
-    draw3d_line_unconditional(v1, v2, lp, lp->l_type);
+    draw3d_line_unconditional(v1, v2, lp, lp->pm3d_color);
 
 }
 
@@ -1022,14 +986,12 @@ draw3d_line (p_vertex v1, p_vertex v2, struct lp_style_type *lp)
 void
 draw3d_point(p_vertex v, struct lp_style_type *lp)
 {
-#ifndef LITE
     /* hidden3d routine can't work if no surface was drawn at all */
     if (hidden3d && draw_surface) {
 	/* Draw vertex as a zero-length edge */
 	draw_line_hidden(v, NULL, lp);
 	return;
     }
-#endif
 
     draw3d_point_unconditional(v, lp);
 }
@@ -1046,10 +1008,8 @@ polyline3d_start(p_vertex v1)
     unsigned int x1, y1;
 
     polyline3d_previous_vertex = *v1;
-#ifndef LITE
     if (hidden3d && draw_surface)
 	return;
-#endif /* LITE */
 
     /* EAM - This may now be unneeded. But I'm not sure. */
     /*       Perhaps the hidden3d code needs the move.   */
@@ -1060,33 +1020,56 @@ polyline3d_start(p_vertex v1)
 void
 polyline3d_next(p_vertex v2, struct lp_style_type *lp)
 {
-    unsigned int x1, y1;
-    unsigned int x2, y2;
+    /* v5: Indicate that line properties are already set so that draw3d_* */
+    /*     routines do not mess up dash patterns by resetting them        */
+    t_colorspec nochange = DEFAULT_COLORSPEC;
 
-    /* Copied from draw3d_line(): */
-#ifndef LITE
-    /* FIXME HBB 20031218: hidden3d mode will still create isolated
-     * edges! */
-    if (hidden3d && draw_surface) {
+    if (hidden3d && draw_surface)
 	draw_line_hidden(&polyline3d_previous_vertex, v2, lp);
-	polyline3d_previous_vertex = *v2;
-	return;
-    }
-#endif
-
-    /* Copied from draw3d_line_unconditional: */
-    /* If use_palette is active, polylines can't be used -->
-     * revert back to old method */
-    if (lp->use_palette) {
+    else
 	draw3d_line_unconditional(&polyline3d_previous_vertex, v2,
-				  lp, lp->l_type);
-	polyline3d_previous_vertex = *v2;
-	return;
+				  lp, nochange);
 
-    }
-
-    TERMCOORD(&polyline3d_previous_vertex, x1, y1);
-    TERMCOORD(v2, x2, y2);
-    draw_clip_line(x1,y1,x2,y2);
     polyline3d_previous_vertex = *v2;
 }
+
+#ifdef NONLINEAR_AXES
+static double
+map_x3d(double x)
+{
+    AXIS *xaxis = &axis_array[FIRST_X_AXIS];
+
+    if (xaxis->linked_to_primary) {
+	xaxis = xaxis->linked_to_primary;
+	x = eval_link_function(xaxis, x);
+    }
+
+    return ((x - xaxis->min)*xscale3d + xcenter3d - 1.0);
+}
+
+static double
+map_y3d(double y)
+{
+    AXIS *yaxis = &axis_array[FIRST_Y_AXIS];
+
+    if (yaxis->linked_to_primary) {
+	yaxis = yaxis->linked_to_primary;
+	y = eval_link_function(yaxis, y);
+    }
+
+    return ((y - yaxis->min)*yscale3d + ycenter3d - 1.0);
+}
+
+static double
+map_z3d(double z)
+{
+    AXIS *zaxis = &axis_array[FIRST_Z_AXIS];
+
+    if (zaxis->linked_to_primary) {
+	zaxis = zaxis->linked_to_primary;
+	z = eval_link_function(zaxis, z);
+    }
+
+    return ((z - floor_z1)*zscale3d + zcenter3d - 1.0);
+}
+#endif
