@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: variable.c,v 1.42.2.1 2011/12/28 19:30:43 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: variable.c,v 1.45 2016/01/10 00:41:13 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - variable.c */
@@ -212,9 +212,7 @@ struct path_table {
     const char *dir;
 };
 
-/* Yet, no special font paths for these operating systems:
- * MSDOS, NeXT, ultrix, VMS, _IBMR2, alliant
- *
+/*
  * Environmental variables are written as $(name).
  * Commands are written as $`command`.
  */
@@ -250,7 +248,7 @@ static const struct path_table fontpath_tbl[] =
 };
 #endif
 
-#if defined(_Macintosh) && !defined(FONTPATHSET)
+#if defined(__APPLE__) && !defined(FONTPATHSET)
 #  define FONTPATHSET
 static const struct path_table fontpath_tbl[] =
 {
